import 'dart:io' show Platform;
import 'package:flutter/material.dart';
import 'package:taboola_sdk/descriptors/device_descriptor.dart';
import 'package:taboola_sdk/standard/webview_manager.dart';
import 'package:taboola_sdk/standard/taboola_webview.dart';
import 'package:taboola_sdk/standard/taboola_standard_listener.dart';
import '../extra_properties_manager.dart';
import 'itemBrowser.dart';
import 'package:taboola_sdk/utils/logger.dart';

class TaboolaStandard extends StatefulWidget {

  ExtraPropertiesManager _extraPropertiesManager;
  final String _publisherId, _mode, _placement, _pageUrl,_pageType, _pageId, _targetType;
  final bool _isFeed;

  final TaboolaStandardListener _taboolaStandardListener;
  ScrollController scrollController = ScrollController();

  TaboolaStandard(this._extraPropertiesManager, this._publisherId, this._mode, this._placement, this._pageUrl, this._pageType, this._pageId,
      this._targetType,this._isFeed, this._taboolaStandardListener);

  @override
  State<StatefulWidget> createState() => TaboolaStandardState();
}

class TaboolaStandardState extends State<TaboolaStandard> with WidgetsBindingObserver {
  WebViewManager _webViewManager;
  TaboolaWebView _taboolaWebView;

  double _webViewHeight;
  bool _didCallScrollListener = false;

  GlobalKey _key = GlobalKey();
  TaboolaStandardState();


  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance.addObserver(this);
    _webViewHeight = 100;

    _webViewManager = new WebViewManager(widget._extraPropertiesManager);
    _taboolaWebView = new TaboolaWebView(
        widget._publisherId,
        widget._placement,
        widget._mode,
        widget._pageType,
        widget._pageUrl,
        widget._pageId,
        widget._targetType,
        _webViewManager);

    _webViewManager.openItemBrowser = openItemBrowser;
    _webViewManager.didResize = taboolaStandardDidResize;
    _webViewManager.didShow = taboolaStandardDidShow;
    _webViewManager.didFailToLoad = widget._taboolaStandardListener.didFailToLoad;
    _webViewManager.didClickOnItem = widget._taboolaStandardListener.didClickOnItem;

    addListenerToScroll();
  }

  @override
  void didUpdateWidget(TaboolaStandard oldWidget) {
    // TODO: implement didUpdateWidget
    super.didUpdateWidget(oldWidget);
    addListenerToScroll();
  }

  @override
  void dispose() {
    WidgetsBinding.instance.removeObserver(this);
    super.dispose();
  }

  void addListenerToScroll() {
    if (widget.scrollController != null) {
      widget.scrollController.addListener(callScrollListener);
    }
  }

  @override
  void didChangeMetrics() {
    // This is a fix for the bug that sometime the listener of the scroll doesnt work
    callScrollListenerIfListenerNotWorking();
  }
  
  void callScrollListener() {
    _didCallScrollListener = true;
    print("Listen to scroll from listener");
    scrollListener();
  }
  
  void callScrollListenerIfListenerNotWorking() {
    if (!_didCallScrollListener) {
      print("Listen to scroll from Metrics");
      scrollListener();
    }
  }

  @override
  Widget build(BuildContext context) {
    return ConstrainedBox(
      key: _key,
      constraints: BoxConstraints(maxHeight: _webViewHeight),
      child: _taboolaWebView,
      );
  }

  void taboolaStandardDidShow(String placement) {
    widget._taboolaStandardListener.didShow(placement);
  }

  void taboolaStandardDidResize(String placement, double height) {
    double maxHeight = DeviceDescriptor.getScreenSize()[1];
    if (maxHeight < height && Platform.isAndroid) {
      height = maxHeight;
    }
    if (_webViewHeight != height) {
      setState(() {
        _webViewHeight = height;
        widget._taboolaStandardListener.didResize(placement,_webViewHeight);
      });
    }

  }

  scrollListener() {
    _taboolaWebView.scrollListener(widget.scrollController, widget._isFeed, _webViewHeight == DeviceDescriptor.getScreenSize()[1]);
  }

  void openItemBrowser(String itemUrl) {
    Logger.log("open item url: $itemUrl");
    Navigator.push(
      context,
      MaterialPageRoute(builder: (context) => ItemBrowser(itemUrl)),
    );
  }

}




